/*  SciGraphica - Scientific graphics and data manipulation
 *  Copyright (C) 2001 Adrian E. Feiguin <feiguin@ifir.edu.ar>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <gtkextra/gtkextra.h>
#include "sg.h"
#include "sg_dialog.h"
#include "sg_ellipse_dialog.h"
#include "sg_stock.h"

typedef struct
{
  GtkWidget *fg_combo;
  GtkWidget *bg_combo;
  GtkWidget *style_combo; 
  GtkWidget *filled_item; 
  GtkWidget *line_width_spin; 
  GtkPlotCanvas *canvas; 
  GtkPlotCanvasEllipse *ellipse;

} SGellipseDialog;

static gchar *line_styles[] = {
  N_("None"),
  N_("Solid"),
  N_("Dotted"),
  N_("Dashed"),
  N_("Dot-dash"),
  N_("Dot-dot-dash"),
  N_("Dot-dash-dash"),
  NULL
};

static void
update_ellipse(SGpropertyDialog *pdialog, gpointer data)
{
  SGellipseDialog *dialog = (SGellipseDialog *)data;
  GdkColor color;
  GtkPlotCanvasEllipse *ellipse;
  GtkWidget *child;
  gint line_style;

  ellipse = dialog->ellipse;

  color = gtk_color_combo_get_selection(GTK_COLOR_COMBO(dialog->fg_combo));

  ellipse->line.color = color;

  color = gtk_color_combo_get_selection(GTK_COLOR_COMBO(dialog->bg_combo));

  ellipse->bg = color;

  ellipse->filled = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->filled_item));

  child = (GtkWidget *)GTK_LIST(GTK_COMBO(dialog->style_combo)->list)->selection->data;
  line_style = gtk_list_child_position(GTK_LIST(GTK_COMBO(dialog->style_combo)->list), child);

  ellipse->line.line_style = (GtkPlotLineStyle)line_style;
  ellipse->line.line_width = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(dialog->line_width_spin));

  gtk_plot_canvas_paint(dialog->canvas);
  gtk_plot_canvas_refresh(dialog->canvas);
}


void
sg_ellipse_dialog (GtkPlotCanvas *canvas, GtkPlotCanvasEllipse *ellipse)
{
  GtkWidget *ref_dialog = NULL;
  SGellipseDialog *dialog;
  GtkWindow *window;
  GtkWidget *frame;
  GtkWidget *table;
  GtkAdjustment *adj;

  dialog = g_new0(SGellipseDialog, 1);
  dialog->canvas = canvas;
  dialog->ellipse = ellipse;

  /* Create widgets */
  frame = sg_property_dialog_new();
  gtk_frame_set_label(GTK_FRAME(frame), _("Properties") );
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
  sg_property_dialog_set_data(SG_PROPERTY_DIALOG(frame), dialog, TRUE);

  table=gtk_table_new(5, 2, FALSE);  
  gtk_container_set_border_width(GTK_CONTAINER(table), 5);
  gtk_table_set_col_spacings(GTK_TABLE(table), 10);
  gtk_table_set_row_spacings(GTK_TABLE(table), 10);

  gtk_container_add(GTK_CONTAINER(frame), table);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new( _("Foreground") ), 0, 1, 0, 1);
  dialog->fg_combo = gtk_color_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->fg_combo, 1, 2, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new( _("Background") ), 0, 1, 1, 2);
  dialog->bg_combo = gtk_color_combo_new();
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->bg_combo, 1, 2, 1, 2);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new( _("Border style") ), 0, 1, 2, 3);
  dialog->style_combo = gtk_combo_new();
  sg_combo_set_items(GTK_COMBO(dialog->style_combo), line_styles);
  gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(dialog->style_combo)->entry), FALSE);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->style_combo, 1, 2, 2, 3);

  gtk_table_attach_defaults(GTK_TABLE(table), 
                            gtk_label_new( _("Border width") ), 0, 1, 3, 4);
  adj = (GtkAdjustment *)gtk_adjustment_new(0., 0., 6., 1., 1., 0.);
  dialog->line_width_spin = gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(dialog->line_width_spin), TRUE);
  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(dialog->line_width_spin), 0);
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->line_width_spin, 1, 2, 3, 4);

  dialog->filled_item = gtk_check_item_new_with_label( _("Fill figure") );
  gtk_table_attach_defaults(GTK_TABLE(table), dialog->filled_item, 0, 1, 4, 5);
  
  /* initialize */

  gtk_list_select_item(GTK_LIST(GTK_COMBO(dialog->style_combo)->list), 
                       ellipse->line.line_style);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(dialog->line_width_spin), 
                            ellipse->line.line_width);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(dialog->filled_item),
                               ellipse->filled);

  sg_color_combo_init(GTK_COLOR_COMBO(dialog->fg_combo), ellipse->line.color); 
  sg_color_combo_init(GTK_COLOR_COMBO(dialog->bg_combo), ellipse->bg); 

  /* connect signals */

  SG_PROPERTY_DIALOG(frame)->ok = update_ellipse;
  SG_PROPERTY_DIALOG(frame)->apply = update_ellipse;
  ref_dialog = sg_dialog_new ( _("SciGraphica: Ellipse "), GTK_ORIENTATION_HORIZONTAL, SG_BUTTON_OK|SG_BUTTON_APPLY|SG_BUTTON_CANCEL, GTK_BUTTONBOX_START);
  window = GTK_WINDOW(ref_dialog);

  gtk_window_set_policy(window, FALSE, FALSE, FALSE);
  sg_dialog_add(ref_dialog, SG_PROPERTY_DIALOG(frame));
  gtk_widget_show_all(frame);
  sg_dialog_run(ref_dialog, GTK_OBJECT(canvas));
}
